@@ -554,7 +554,7 @@ gs_lock_plug_run (GSLockPlug *plug)
                 g_signal_handler_disconnect (plug, unmap_handler);
                 g_signal_handler_disconnect (plug, delete_handler);
                 g_signal_handler_disconnect (plug, destroy_handler);
-                g_signal_handler_disconnect (plug, keymap_handler);
+                g_signal_handler_disconnect (keymap, keymap_handler);
         }
 
         g_object_unref (plug);
@@ -1300,7 +1300,7 @@ gs_lock_plug_set_busy (GSLockPlug *plug)
 
         cursor = gdk_cursor_new (GDK_WATCH);
         gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
-        gdk_cursor_unref (cursor);
+        g_object_unref (cursor);
 }
 
 void
@@ -1313,7 +1313,7 @@ gs_lock_plug_set_ready (GSLockPlug *plug)
 
         cursor = gdk_cursor_new (GDK_LEFT_PTR);
         gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
-        gdk_cursor_unref (cursor);
+        g_object_unref (cursor);
 }
 
 void
@@ -1561,8 +1561,7 @@ create_page_one (GSLockPlug *plug)
 
         hbox_user = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
         gtk_box_pack_start (GTK_BOX (plug->priv->vbox), hbox_user, FALSE, FALSE, 0);
-        gtk_misc_set_alignment (GTK_MISC (hbox_user), 0.5, 0.5);
-                       
+
         plug->priv->auth_face_image = gtk_image_new ();
         gtk_box_pack_start (GTK_BOX (hbox_user), plug->priv->auth_face_image, TRUE, TRUE, 0);
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 1, 0.5);
